@@ -15,7 +15,6 @@ use POSIX qw/strftime/;
 use Digest::SHA qw/sha256_hex/;
 use File::Slurp;
 use Carp;
-use Data::Dumper;
 
 =head1 NAME
 
@@ -23,12 +22,11 @@ Net::Amazon::Glacier - An implementation of the Amazon Glacier RESTful API.
 
 =head1 VERSION
 
-Version 0.12
+Version 0.13
 
 =cut
 
-our $VERSION = '0.12';
-
+our $VERSION = '0.13';
 
 =head1 SYNOPSIS
 
@@ -43,8 +41,42 @@ Perhaps a little code snippet.
 		'AKIMYACCOUNTID',
 		'MYSECRET',
 	);
+	
+	my $vault = 'a_vault';
+	my @vaults = $glacier->list_vaults();
+	
+	if ( $glacier->create_vault( $vault ) ) {
 
-	$glacier->create_vault( 'a_vault' );
+		if ( my $archive_id = $glacier->upload_archive( './archive.7z' ) ) {
+
+			my $job_id = $glacier->inititate_job( $vault, $archive_id );
+			
+			# Jobs generally take about 4 hours to complete
+			my $job_description = $glacier->describe_job( $vault, $job_id );
+			
+			# For a better way to wait for completion, see
+			# http://docs.aws.amazon.com/amazonglacier/latest/dev/api-initiate-job-post.html
+			while ( $job_description->{'StatusCode'} ne 'Succeeded' ) {
+				sleep 15 * 60 * 60;
+				$job_description = $glacier->describe_job( $vault, $job_id );
+			}
+			
+			my $archive_bytes = $glacier->get_job_output( $vault, $job_id );
+			
+			# Jobs live as completed jobs for "a period", according to
+			# http://docs.aws.amazon.com/amazonglacier/latest/dev/api-jobs-get.html
+			my @jobs = $glacier->list_jobs( $vault );
+			
+			# As of 2013-02-09 jobs are blindly created even if a job for the same archive_id and Range exists.
+			# Keep $archive_ids, reuse the expensive job resource, and remember 4 hours.
+			foreach my $job ( @jobs ) {
+				next unless $job->{ArchiveId} eq $archive_id;
+				my $archive_bytes = $glacier->get_job_output( $vault, $job_id );
+			}
+
+		}
+		
+	}
 
 The functions are intended to closely reflect Amazon's Glacier API. Please see Amazon's API reference for documentation of the functions: L<http://docs.amazonwebservices.com/amazonglacier/latest/dev/amazon-glacier-api.html>.
 
@@ -70,7 +102,7 @@ sub new {
 =head2 create_vault( $vault_name )
 
 Creates a vault with the specified name. Returns true on success, false on failure.
-
+L<Create Vault (PUT vault)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-put.html>
 =cut
 
 sub create_vault {
@@ -83,7 +115,7 @@ sub create_vault {
 =head2 delete_vault( $vault_name )
 
 Deletes the specified vault. Returns true on success, false on failure.
-
+L<Delete Vault (DELETE vault)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-delete.html>
 =cut
 
 sub delete_vault {
@@ -96,6 +128,7 @@ sub delete_vault {
 =head2 describe_vault( $vault_name )
 
 Fetches information about the specified vault. Returns a hash reference with the keys described by L<http://docs.amazonwebservices.com/amazonglacier/latest/dev/api-vault-get.html>. Returns false on failure.
+L<Describe Vault (GET vault)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-get.html>
 
 =cut
 
@@ -106,26 +139,108 @@ sub describe_vault {
 	return $self->_decode_and_handle_response( $res );
 }
 
-=head2 list_vaults( [$limit, [$marker] ]  )
+=head2 list_vaults
 
-Lists the vaults. Returns a hash reference with a "marker" key, for pagination, and a "VaultList", which describes the vaults. The content of the vault list, and how $limit and $marker can be used for pagination, is described by L<http://docs.amazonwebservices.com/amazonglacier/latest/dev/api-vaults-get.html>. Returns false on failure.
+Lists the vaults. Returns an array with all vaults.
+L<Amazon Glacier List Vaults (GET vaults)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-vaults-get.html>.
+
+A call to list_vaults can result in many calls to the the Amazon API at a rate of 1 per 1,000 vaults in existence.
+Calls to List Vaults in the API are L<free|http://aws.amazon.com/glacier/pricing/#storagePricing>.
 
 =cut
 
 sub list_vaults {
-	my ( $self, $limit, $marker ) = @_;
-	$limit //= 1000; # max and default
-	my $uri = "/-/vaults?limit=$limit";
-	$uri .= "&marker=$marker" if defined $marker;
-	my $res = $self->_send_receive( GET => $uri );
+	my ( $self ) = @_;
+	my @vaults;
+	my $marker;
+	do {
+		#1000 is the default limit, send a marker if needed
+		my $res = $self->_send_receive( GET => "/-/vaults?limit=1000" . $marker?'&'.$marker:'');
+		my $decoded = $self->_decode_and_handle_response( $res );
+
+		push @vaults, @{$decoded->{VaultList}};
+		$marker = $decoded->{Marker};
+	} while ( $marker );
+	return ( \@vaults );
+}
+
+=head2 set_vault_notifications( $vault_name, $sns_topic, $events )
+
+Sets vault notifications for a given vault.
+An SNS Topic to send notifications to must be provided. The SNS Topic must grant permission to the vault to be allowed to publish notifications to the topic.
+An array ref to a list of events must be provided. Valid events are ArchiveRetrievalCompleted and InventoryRetrievalCompleted
+upon job completion may also be supplied.
+Return true on success, false otherwise.
+L<Set Vault Notification Configuration (PUT notification-configuration)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-notifications-put.html>.
+
+=cut
+
+sub set_vault_notifications {
+	my ( $self, $vault_name, $sns_topic, $events ) = @_;
+	croak "no vault name given" unless $vault_name;
+	croak "no sns topic given" unless $sns_topic;
+	croak "events should be an array ref" unless ref $events eq 'ARRAY';
+	
+	my $content_raw;
+	
+	$content_raw->{SNSTopic} = $sns_topic
+		if defined($sns_topic);
+	
+	$content_raw->{Events} = $events
+		if defined($events);
+	
+	my $res = $self->_send_receive(
+		PUT => "/-/vaults/$vault_name/notification-configuration",
+		[
+		],
+		encode_json($content_raw),
+	);
+	return $res->is_success;
+}
+
+=head2 get_vault_notifications( $vault_name )
+
+Gets vault notifications status for a given vault.
+Return false on failure or a hash with an 'SNSTopic' and and array of 'Events' on success.
+L<Get Vault Notifications (GET notification-configuration)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-notifications-get.html>.
+
+=cut
+
+sub get_vault_notifications {
+	my ( $self, $vault_name, $sns_topic, $events ) = @_;
+	croak "no vault name given" unless $vault_name;
+	
+	my $res = $self->_send_receive(
+		PUT => "/-/vaults/$vault_name/notification-configuration",
+	);
+	return 0 unless $res->is_success;
 	return $self->_decode_and_handle_response( $res );
 }
 
+=head2 delete_vault_notifications( $vault_name )
+
+Deletes vault notifications for a given vault.
+Return true on success, false otherwise.
+L<Delete Vault Notifications (DELETE notification-configuration)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-notifications-delete.html>.
+
+=cut
+
+sub delete_vault_notifications {
+	my ( $self, $vault_name, $sns_topic, $events ) = @_;
+	croak "no vault name given" unless $vault_name;
+	
+	my $res = $self->_send_receive(
+		DELETE => "/-/vaults/$vault_name/notification-configuration",
+	);
+	return $res->is_success;
+}
+
 =head1 ARCHIVE OPERATIONS
 
 =head2 upload_archive( $vault_name, $archive_path, [ $description ] )
 
 Uploads an archive to the specified vault. $archive_path is the local path to any file smaller than 4GB. For larger files, see multi-part upload. An archive description of up to 1024 printable ASCII characters can be supplied. Returns the Amazon-generated archive ID on success, or false on failure.
+L<Upload Archive (POST archive)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-archive-post.html>
 
 =cut
 
@@ -144,7 +259,7 @@ sub upload_archive {
 	$th->calc_tree;
 
 	my $res = $self->_send_receive(
-		POST => "/-/vaults/$vault_name/archives", 
+		POST => "/-/vaults/$vault_name/archives",
 		[
 			'x-amz-archive-description' => $description,
 			'x-amz-sha256-tree-hash' => $th->get_final_hash(),
@@ -162,6 +277,20 @@ sub upload_archive {
 	}
 }
 
+=head2 delete_archive( $vault_name, $archive_id )
+
+Issues a request to delete a file from Glacier. $archive_id is the ID you received either when you uploaded the file originally or from an inventory.
+L<Delete Archive (DELETE archive)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-archive-delete.html>
+
+=cut
+
+sub delete_archive {
+	my ( $self, $vault_name, $archive_id ) = @_;
+	croak "no vault name given" unless $vault_name;
+	croak "no archive ID given" unless $archive_id;
+	my $res = $self->_send_receive( DELETE => "/-/vaults/$vault_name/archives/$archive_id" );
+	return $res->is_success;
+}
 
 =head1 JOB OPERATIONS
 
@@ -172,6 +301,7 @@ Initiates an archive retrieval job. $archive_id is an ID previously
 retrieved from Amazon Glacier. A job description of up to 1,024 printable
 ASCII characters may be supplied. An SNS Topic to send notifications to
 upon job completion may also be supplied.
+L<Initiate a Job (POST jobs)|docs.aws.amazon.com/amazonglacier/latest/dev/api-initiate-job-post.html#api-initiate-job-post-requests-syntax>.
 
 =cut
 
@@ -207,6 +337,7 @@ $sns_topic ] )
 Initiates an inventory retrieval job. $format is either CSV or JSON (default).
 A job description of up to 1,024 printable ASCII characters may be supplied. An
 SNS Topic to send notifications to upon job completion may also be supplied.
+L<Initiate a Job (POST jobs)|docs.aws.amazon.com/amazonglacier/latest/dev/api-initiate-job-post.html#api-initiate-job-post-requests-syntax>.
 
 =cut
 
@@ -237,10 +368,38 @@ sub initiate_inventory_retrieval {
 	return $res->header('x-amz-job-id');
 }
 
+=head2 initiate_job( ( $vault_name, $archive_id, [
+$description, $sns_topic ] )
+
+Effectively calls initiate_inventory_retrieval.
+Exists for the sole purpose or implementing the Amazon Glacier Developer Guide (API Version 2012-06-01)
+nomenclature.
+L<Initiate a Job (POST jobs)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-initiate-job-post.html>.
+
+=cut
+
+sub initiate_job {
+	initiate_inventory_retrieval( @_ );
+}
+
+=head2 describe_job( $vault_name, $job_id )
+
+Retrieves a hashref with information about the requested JobID.
+L<Amazon Glacier Describe Job (GET JobID)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-describe-job-get.html>.
+
+=cut
+
+sub describe_job {
+	my ( $self, $vault_name, $job_id ) = @_;
+	my $res = $self->_send_receive( GET => "/-/vaults/$vault_name/jobs/$job_id" );
+	return $self->_decode_and_handle_response( $res );
+}
+
 =head2 get_job_output( $vault_name, $job_id, [ $range ] )
 
 Retrieves the output of a job, returns a binary blob. Optional range
 parameter is passed as an HTTP header.
+L<Amazon Glacier Get Job Output (GET output)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-job-output-get.html>.
 
 =cut
 
@@ -260,20 +419,31 @@ sub get_job_output {
 	}
 }
 
-=head2 describe_job( $vault_name, $job_id )
+=head2 list_jobs( $vault_name )
 
-Retrieves a hashref with information about the requested JobID
+Return an array with information about all recently completed jobs for the specified vault.
+L<Amazon Glacier List Jobs (GET jobs)|http://docs.aws.amazon.com/amazonglacier/latest/dev/api-jobs-get.html>.
+
+A call to list_jobs can result in many calls to the the Amazon API at a rate of 1 per 1,000 recently completed job in existence.
+Calls to List Jobs in the API are L<free|http://aws.amazon.com/glacier/pricing/#storagePricing>.
 
 =cut
 
-sub describe_job {
-	my ( $self, $vault_name, $job_id ) = @_;
-	my $res = $self->_send_receive( GET => "/-/vaults/$vault_name/jobs/$job_id" );
-	return $self->_decode_and_handle_response( $res );
+sub list_jobs {
+	my ( $self, $vault_name ) = @_;
+	my @completed_jobs;
+	my $marker;
+	do {
+		#1000 is the default limit, send a marker if needed
+		my $res = $self->_send_receive( GET => "/-/vaults/$vault_name/jobs?limit=1000" . $marker?'&'.$marker:'' );
+		my $decoded = $self->_decode_and_handle_response( $res );
+		
+		push @completed_jobs, @{$decoded->{JobList}};
+		$marker = $decoded->{Marker};
+	} while ( $marker );
+	return ( \@completed_jobs );
 }
 
-
-
 # helper functions
 
 sub _decode_and_handle_response {
@@ -322,10 +492,6 @@ The following parts of Amazon's API have not yet been implemented. This is mainl
 
 =over 4
 
-=item * PUT/GET/DELETE vault notifications
-
-=item * Archive deletion
-
 =item * Multipart upload operations
 
 =back
@@ -336,7 +502,7 @@ See also Victor Efimov's MT::AWS::Glacier, an application for AWS Glacier synchr
 
 =head1 AUTHORS
 
-Written and maintained by Tim Nordenfur, C<< <tim at gurka.se> >>. Support for job operations was contributed by Ted Reed at IMVU.
+Maintained and originally written by Tim Nordenfur, C<< <tim at gurka.se> >>. Support for job operations was contributed by Ted Reed at IMVU. Support for many operations was contributed by Gonzalo Barco.
 
 =head1 BUGS
 
